@@ -10,6 +10,9 @@
 #include <stddef.h>
 #include <assert.h>
 #include <errno.h>
+#if HAVE_HEADER_INTTYPES_H
+#include <inttypes.h>
+#endif
 
 // TODO: fix this to make it crosscompile-friendly: RZ_SYS_OSTYPE ?
 /* operating system */
@@ -29,29 +32,25 @@
 #define RZ_MODE_CLASSDUMP 0x040
 #define RZ_MODE_EQUAL     0x080
 
-/**
- * \brief Enum to describe the way data are printed
- */
-typedef enum {
-	RZ_OUTPUT_MODE_STANDARD = 1 << 0,
-	RZ_OUTPUT_MODE_JSON = 1 << 1,
-	RZ_OUTPUT_MODE_RIZIN = 1 << 2,
-	RZ_OUTPUT_MODE_QUIET = 1 << 3,
-	RZ_OUTPUT_MODE_SDB = 1 << 4,
-	RZ_OUTPUT_MODE_LONG = 1 << 5,
-	RZ_OUTPUT_MODE_LONG_JSON = 1 << 6,
-	RZ_OUTPUT_MODE_TABLE = 1 << 7,
-	RZ_OUTPUT_MODE_QUIETEST = 1 << 8,
-} RzOutputMode;
+#define RZ_IN    /* do not use, implicit */
+#define RZ_OUT   /* parameter is written, not read */
+#define RZ_INOUT /* parameter is read and written */
 
-#define RZ_IN        /* do not use, implicit */
-#define RZ_OUT       /* parameter is written, not read */
-#define RZ_INOUT     /* parameter is read and written */
+#ifdef RZ_BINDINGS
+#define RZ_OWN    __attribute__((annotate("RZ_OWN")))
+#define RZ_BORROW __attribute__((annotate("RZ_BORROW")))
+
+#define RZ_NONNULL   __attribute__((annotate("RZ_NONNULL")))
+#define RZ_NULLABLE  __attribute__((annotate("RZ_NULLABLE")))
+#define RZ_DEPRECATE __attribute__((annotate("RZ_DEPRECATE")))
+#else
 #define RZ_OWN       /* pointer ownership is transferred */
 #define RZ_BORROW    /* pointer ownership is not transferred, it must not be freed by the receiver */
 #define RZ_NONNULL   /* pointer can not be null */
 #define RZ_NULLABLE  /* pointer can be null */
 #define RZ_DEPRECATE /* should not be used in new code and should/will be removed in the future */
+#endif
+
 #define RZ_IFNULL(x) /* default value for the pointer when null */
 #ifdef __GNUC__
 #define RZ_UNUSED __attribute__((__unused__))
@@ -181,8 +180,6 @@ typedef enum {
 #include <rz_types_base.h>
 #include <rz_constructor.h>
 
-#undef _FILE_OFFSET_BITS
-#define _FILE_OFFSET_BITS 64
 #undef _GNU_SOURCE
 #define _GNU_SOURCE
 
@@ -244,8 +241,8 @@ typedef int (*PrintfCallback)(const char *str, ...) RZ_PRINTF_CHECK(1, 2);
 #ifdef RZ_API
 #undef RZ_API
 #endif
-#if RZ_SWIG
-#define RZ_API export
+#ifdef RZ_BINDINGS
+#define RZ_API __attribute__((annotate("RZ_API")))
 #elif RZ_INLINE
 #define RZ_API inline
 #else
@@ -273,7 +270,7 @@ typedef int (*PrintfCallback)(const char *str, ...) RZ_PRINTF_CHECK(1, 2);
 #define RZ_NEW(x)        (x *)malloc(sizeof(x))
 #define RZ_NEWCOPY(x, y) (x *)rz_new_copy(sizeof(x), y)
 
-static inline void *rz_new_copy(int size, void *data) {
+static inline void *rz_new_copy(int size, const void *data) {
 	void *a = malloc(size);
 	if (a) {
 		memcpy(a, data, size);
@@ -296,11 +293,12 @@ static inline void *rz_new_copy(int size, void *data) {
 	((char *)(((size_t)(v) + (t - 1)) & ~(t - 1)))
 
 #define RZ_BIT_MASK32(x, y) ((1UL << (x)) - (1UL << (y)))
+#define RZ_BIT_MASK64(x, y) ((1ULL << (x)) - (1ULL << (y)))
 #define RZ_BIT_SET(x, y)    (((ut8 *)x)[y >> 4] |= (1 << (y & 0xf)))
 #define RZ_BIT_UNSET(x, y)  (((ut8 *)x)[y >> 4] &= ~(1 << (y & 0xf)))
 #define RZ_BIT_TOGGLE(x, y) (RZ_BIT_CHK(x, y) ? RZ_BIT_UNSET(x, y) : RZ_BIT_SET(x, y))
 
-//#define RZ_BIT_CHK(x,y) ((((const ut8*)x)[y>>4] & (1<<(y&0xf))))
+// #define RZ_BIT_CHK(x,y) ((((const ut8*)x)[y>>4] & (1<<(y&0xf))))
 #define RZ_BIT_CHK(x, y) (*(x) & (1 << (y)))
 
 /* try for C99, but provide backwards compatibility */
@@ -347,7 +345,7 @@ static inline void *rz_new_copy(int size, void *data) {
 #if !(defined(__GNUC__) && __GNUC__ < 5) || defined(__clang__)
 #define rz_offsetof(type, member) offsetof(type, member)
 #else
-#if __SDB_WINDOWS__
+#if __WINDOWS__
 #define rz_offsetof(type, member) ((unsigned long)(ut64) & ((type *)0)->member)
 #else
 #define rz_offsetof(type, member) ((unsigned long)&((type *)0)->member)
@@ -367,27 +365,29 @@ static inline void *rz_new_copy(int size, void *data) {
 	}
 
 #if __WINDOWS__
-#define PFMT64x "I64x"
-#define PFMT64d "I64d"
-#define PFMT64u "I64u"
-#define PFMT64o "I64o"
-#define PFMTSZx "Ix"
-#define PFMTSZd "Id"
-#define PFMTSZu "Iu"
-#define PFMTSZo "Io"
-#define LDBLFMT "f"
-#define HHXFMT  "x"
+#define PFMT64x  "I64x"
+#define PFMT64d  "I64d"
+#define PFMT64u  "I64u"
+#define PFMT64o  "I64o"
+#define PFMTSZx  "Ix"
+#define PFMTSZd  "Id"
+#define PFMTSZu  "Iu"
+#define PFMTSZo  "Io"
+#define LDBLFMTg "g"
+#define LDBLFMTf "f"
+#define HHXFMT   "x"
 #else
-#define PFMT64x "llx"
-#define PFMT64d "lld"
-#define PFMT64u "llu"
-#define PFMT64o "llo"
-#define PFMTSZx "zx"
-#define PFMTSZd "zd"
-#define PFMTSZu "zu"
-#define PFMTSZo "zo"
-#define LDBLFMT "Lf"
-#define HHXFMT  "hhx"
+#define PFMT64x  "llx"
+#define PFMT64d  "lld"
+#define PFMT64u  "llu"
+#define PFMT64o  "llo"
+#define PFMTSZx  "zx"
+#define PFMTSZd  "zd"
+#define PFMTSZu  "zu"
+#define PFMTSZo  "zo"
+#define LDBLFMTg "Lg"
+#define LDBLFMTf "Lf"
+#define HHXFMT   "hhx"
 #endif
 
 #define PFMTDPTR "td"
@@ -549,14 +549,15 @@ typedef enum {
 	RZ_SYS_ARCH_HPPA,
 	RZ_SYS_ARCH_V810,
 	RZ_SYS_ARCH_LM32,
-	RZ_SYS_ARCH_RISCV
+	RZ_SYS_ARCH_RISCV,
+	RZ_SYS_ARCH_TRICORE,
 } RzSysArch;
 
 /* os */
 #if defined(__QNX__)
 #define RZ_SYS_OS "qnx"
-//#elif TARGET_OS_IPHONE
-//#define RZ_SYS_OS "ios"
+// #elif TARGET_OS_IPHONE
+// #define RZ_SYS_OS "ios"
 #elif defined(__APPLE__)
 #define RZ_SYS_OS "darwin"
 #elif defined(__linux__)
@@ -662,4 +663,6 @@ typedef int RzRef;
 		rz_unref(x, n##_free); \
 	}
 
+typedef struct rz_core_t RzCore;
+
 #endif // RZ_TYPES_H
